/*
 * Copyright Bigeon Emmanuel (2014)
 *
 * emmanuel@bigeon.fr
 *
 * This software is a computer program whose purpose is to
 * provide a generic framework for console applications.
 *
 * This software is governed by the CeCILL license under French law and
 * abiding by the rules of distribution of free software.  You can  use,
 * modify and/or redistribute the software under the terms of the CeCILL
 * license as circulated by CEA, CNRS and INRIA at the following URL
 * "http://www.cecill.info".
 *
 * As a counterpart to the access to the source code and  rights to copy,
 * modify and redistribute granted by the license, users are provided only
 * with a limited warranty  and the software's author,  the holder of the
 * economic rights,  and the successive licensors  have only  limited
 * liability.
 *
 * In this respect, the user's attention is drawn to the risks associated
 * with loading,  using,  modifying and/or developing or reproducing the
 * software by the user in light of its specific status of free software,
 * that may mean  that it is complicated to manipulate,  and  that  also
 * therefore means  that it is reserved for developers  and  experienced
 * professionals having in-depth computer knowledge. Users are therefore
 * encouraged to load and test the software's suitability as regards their
 * requirements in conditions enabling the security of their systems and/or
 * data to be ensured and,  more generally, to use and operate it in the
 * same conditions as regards security.
 *
 * The fact that you are presently reading this means that you have had
 * knowledge of the CeCILL license and that you accept its terms.
 */
/**
 * gclc:net.bigeon.gclc.ConsoleApplicationTest.java
 * Created on: Jun 9, 2016
 */
package net.bigeon.gclc;

/*-
 * #%L
 * Generic Command Ligne console
 * %%
 * Copyright (C) 2014 - 2018 bigeon.fr
 * %%
 * This software is governed by the CeCILL  license under French law and
 * abiding by the rules of distribution of free software.  You can  use,
 * modify and/ or redistribute the software under the terms of the CeCILL
 * license as circulated by CEA, CNRS and INRIA at the following URL
 * "http://www.cecill.info".
 *
 * As a counterpart to the access to the source code and  rights to copy,
 * modify and redistribute granted by the license, users are provided only
 * with a limited warranty  and the software's author,  the holder of the
 * economic rights,  and the successive licensors  have only  limited
 * liability.
 *
 * In this respect, the user's attention is drawn to the risks associated
 * with loading,  using,  modifying and/or developing or reproducing the
 * software by the user in light of its specific status of free software,
 * that may mean  that it is complicated to manipulate,  and  that  also
 * therefore means  that it is reserved for developers  and  experienced
 * professionals having in-depth computer knowledge. Users are therefore
 * encouraged to load and test the software's suitability as regards their
 * requirements in conditions enabling the security of their systems and/or
 * data to be ensured and,  more generally, to use and operate it in the
 * same conditions as regards security.
 *
 * The fact that you are presently reading this means that you have had
 * knowledge of the CeCILL license and that you accept its terms.
 * #L%
 */
import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertFalse;
import static org.junit.Assert.assertNotNull;
import static org.junit.Assert.assertTrue;
import static org.junit.Assert.fail;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.PipedInputStream;
import java.io.PipedOutputStream;
import java.io.PrintStream;
import java.nio.charset.StandardCharsets;

import org.junit.Test;

import net.bigeon.gclc.command.ICommand;
import net.bigeon.gclc.command.base.ExitCommand;
import net.bigeon.gclc.exception.CommandRunException;
import net.bigeon.gclc.exception.CommandRunExceptionType;
import net.bigeon.gclc.exception.InvalidCommandName;
import net.bigeon.gclc.i18n.Messages;
import net.bigeon.gclc.manager.ConsoleInput;
import net.bigeon.gclc.manager.ConsoleOutput;
import net.bigeon.gclc.manager.PipedConsoleInput;
import net.bigeon.gclc.manager.PipedConsoleOutput;

/** Test class for ConsoleApplication
 *
 * @author Emmanuel Bigeon */
@SuppressWarnings({ "javadoc", "nls", "static-method" })
public class ConsoleApplicationTest {

    /** 3 seconds in milliseconds */
    protected static final long THREE_SECONDS = 3000;

    /** Test the base of a console application */
    @Test
    public void testConsoleApplication() {

        try (PipedOutputStream pout = new PipedOutputStream();
                PipedInputStream pis = new PipedInputStream(pout);
                BufferedReader buf = new BufferedReader(
                        new InputStreamReader(pis, StandardCharsets.UTF_8));
                PipedConsoleInput in = new PipedConsoleInput(new PrintStream(pout))) {
            final ConsoleApplication app = new ConsoleApplication(null, in, "", "");
            app.exit();
        } catch (final IOException e) {
            fail("System Console Manager failed");
        }

    }

    @Test
    public void testExecution()
            throws IOException, InterruptedException, InvalidCommandName {
        try (CommandTestingApplication application = new CommandTestingApplication()) {

            // remove welcome
            assertEquals("Header should be preserved",
                    application.getApplication().header, application.readNextLine());
            // Remove first prompt
            application.sendCommand("");
            application.sendCommand("test");
            assertEquals("Test should run", "Test command ran fine",
                    application.readNextLine());
            application.sendCommand("toto");
            assertEquals("Command fail should dispaly appropriate message",
                    Messages.getString("ConsoleApplication.cmd.failed", "toto"),
                    application.readNextLine());
            assertEquals("Unrecognized comment should result in a specific message.",
                    Messages.getString("CommandProvider.unrecognized", "toto"),
                    application.readNextLine());
            application.sendCommand("long");
            assertEquals("Before wait should receive message", "Waita minute",
                    application.readNextLine());
            assertEquals("Unexpected message", "done!", application.readNextLine());

            final CommandRequestListener crl = new CommandRequestListener() {

                @Override
                public void commandRequest(final String command) {
                    //
                }
            };
            final CommandRequestListener crl2 = new CommandRequestListener() {

                @Override
                public void commandRequest(final String command) {
                    //
                }
            };
            application.getApplication().addListener(crl);
            application.getApplication().addListener(crl2);
            application.sendCommand("test");
            assertEquals("Unexpected message", "Test command ran fine",
                    application.readNextLine());
            application.getApplication().removeListener(crl2);
            application.getApplication().removeListener(crl);
            application.getApplication().removeListener(crl);

            assertTrue("Unclosed application should be running",
                    application.getApplication().isRunning());

            application.sendCommand("exit");
            assertEquals("Footer should be preserved",
                    application.getApplication().footer, application.readNextLine());
            assertFalse("Stopped application should not be running",
                    application.getApplication().isRunning());
        }

        ConsoleApplication appli = null;
        try (PipedConsoleOutput manager = new PipedConsoleOutput();
                PipedOutputStream pout = new PipedOutputStream();
                PipedInputStream pis = new PipedInputStream(pout);
                BufferedReader buf = new BufferedReader(
                        new InputStreamReader(pis, StandardCharsets.UTF_8));
                PipedConsoleInput in = new PipedConsoleInput(new PrintStream(pout))) {
            final ConsoleApplication app = new ConsoleApplication(manager, in, null,
                    null);
            appli = app;
            app.add(new ExitCommand("exit", app));

            final Thread th = new Thread(new Runnable() {

                @Override
                public void run() {
                    app.start();
                }
            });
            th.start();
            in.type("exit");
            th.join();
        }

        assertNotNull(
                "Application should still exist even if the console input and output are closed.",
                appli);
        appli.start();
        assertFalse("Application should not start on closed console input and output",
                appli.isRunning());
    }

    @Test
    public void testInterpretCommand() throws InvalidCommandName, IOException {
        try (PipedConsoleInput test = new PipedConsoleInput(null);
                PipedConsoleOutput out = new PipedConsoleOutput()) {
            final ConsoleApplication appl = new ConsoleApplication(out, test, "", "");

            appl.interpretCommand("invalid cmd \"due to misplaced\"quote");
            assertEquals("Specific error message expected",
                    "Command line cannot be parsed", out.readNextLine());

            appl.interpretCommand("");

            final String message = "message";
            appl.add(new ICommand() {

                /* (non-Javadoc)
                 * @see net.bigeon.gclc.command.ICommand#execute(net.bigeon.gclc.
                 * manager.ConsoleOutput, net.bigeon.gclc.manager.ConsoleInput,
                 * java.lang.String[]) */
                @Override
                public void execute(final ConsoleOutput out, final ConsoleInput in,
                        final String... args) throws CommandRunException {
                    throw new CommandRunException(CommandRunExceptionType.USAGE, message);
                }

                @Override
                public String getCommandName() {
                    return "fail";
                }

                @Override
                public void help(final ConsoleOutput manager, final String... args)
                        throws IOException {
                    manager.println(message);
                }

                @Override
                public String tip() {
                    return "";
                }

            });
            appl.interpretCommand("fail");
            assertEquals("Unexpected message",
                    Messages.getString("ConsoleApplication.cmd.failed", "fail"),
                    out.readNextLine());
            assertEquals("Unexpected message", message, out.readNextLine());
            assertEquals("Unexpected message", message, out.readNextLine());

        }
    }
}
